import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';

export default class BarGraphComponent extends Component {
  @tracked lineElement;

  get isLoaded() {
    return (this.args.dataset || this.args.dataset === []) && !!this.args.dimensions;
  }

  get fill() {
    return this.args.fill ? this.args.fill : 'none'
  }

  get color() {
    return this.args.color ? this.args.color : 'black'
  }

  get opacity() {
    return this.args.opacity ? this.args.opacity : '0.6'
  }

  get strokeWidth() {
    return this.args.strokeWidth ? this.args.strokeWidth : '0'
  }

  get strokeColor() {
    return this.args.strokeColor ? this.args.strokeColor : this.color
  }

  get xKey() {
    return this.args.xKey ? this.args.xKey : 'at';
  }

  get yKey() {
    return this.args.yKey ? this.args.yKey : 'value';
  }

  get barWidth() {
    return this.args.barWidth
      ? this.args.barWidth
      : this.autoWidth()
  }

  get aggregatedTime() {
    return this.args.aggregatedTime ? this.args.aggregatedTime : 60 * 60 * 1000;
  }

  get barAnchor() { // Anchor bar to start of timestamp, or center it ("center", "left")
    return this.args.barAnchor ? this.args.barAnchor : "left"
  }

  autoWidth() {
    if((!this.args.dataset && !this.args.range) || !this.args.dimensions.width) return '5';
    return this.args.xScale(new Date(this.args.range[0].getTime() + this.aggregatedTime)) - this.args.xScale(this.args.range[0]);
  };

  get bars() {
    const { xKey, yKey, barWidth } = this;
    const zero =  this.args.dimensions.height - this.args.yScale(0)
    return this.args.dataset.map((datapoint, index) => {
      const timeLocation = this.args.xScale(datapoint[xKey]);
      const valueHeight = this.args.yScale(datapoint[yKey]);
      return {
        index,
        value: datapoint[yKey],
        time: datapoint[xKey],
        barWidth,
        barHeight: this.args.dimensions.height - (valueHeight + zero),
        barY: valueHeight,
        barX:  this.barAnchor === "left"
          ? timeLocation
          : timeLocation - (barWidth / 2)
      }
    })
  }
}